#
# poly500.R,  2 Apr 23
#
# Data from:
# Measuring Failed Disruption Propagation in Genetic Programming
# William B. Langdon and Afnan Al-Subaihin and David Clark
# Kindly provided by Bill Langdon

library("colorspace")
library("plyr")


par(bty="l")
par(las=1)
par(pch="+")
pal_col=rainbow(10)

distance_cnt=function(df)
{
points(df$distance, df$count, col=pal_col[df$run[1]])
}


p500=read.csv("poly-p500.csv")

p500=subset(p500, (fraction > 0) & (fraction < 1))
p500=subset(p500, count > 0)

p500$tree_size=round(p500$count/p500$fraction)

plot(~ log(tree_size)+log(distance)+log(count)+testind+run, data=p500)

# Plot used in blog post
plot(1, type="n", log="y",
	xaxs="i",
	xlim=c(1, 600), ylim=c(1, 60000),
	xlab="Distance", ylab="Synchronising runs")

r9=subset(p500, run == 9)
distance_cnt(r9)
# r2=subset(p500, run == 2)
# distance_cnt(r2)
r1=subset(p500, run == 1)
distance_cnt(r1)

dev.copy(dev=png, file="GI-runs-distance.png")
dev.off()

d_ply(p500, .(run), distance_cnt)

frac_mod=glm(fraction ~ log(distance)+log(tree_size), data=p500)
summary(frac_mod)
dist_mod=glm(log(distance) ~ fraction+tree_size, data=p500)
summary(dist_mod)

# poly_mod=glm(log(count) ~ log(distance)+sqrt(tree_size), data=p500)
poly_mod=glm(log(distance) ~ count*log(tree_size), data=p500)
summary(poly_mod)


# Parse tree plot generation from stackoverflow post
# https://stackoverflow.com/questions/33419325/plotting-expression-trees-in-r

## https://github.com/halpo/parser/blob/master/R/plot.parser.R
## Modified slightly to return graph instead of print/add attr
parser2graph <- function(y, ...){
    y$new.id <- seq_along(y$id)
    h <- graph.tree(0) + vertices(id = y$id, label= y$text)
    for(i in 1:nrow(y)){
        if(y[i, 'parent'])
            h <- h + edge(c(y[y$id == y[i, 'parent'], 'new.id'], y[i, 'new.id']))
    }
    h <- set_edge_attr(h, 'color', value='black')
    return(h)
}

The next function collapses the parse tree by removing all the '(){}' and remaining gaps. The idea is to first move all the labels up one level in the tree, then clip the leaves. And finally all the gaps from nested expressions ('(){}') are removed by creating/destroying edges. I colored the edges blue where levels of nesting from brackets/braces were removed.

## Function to collapse the parse tree (removing () and {})
parseTree <- function(string, ignore=c('(',')','{','}'), ...) {
    dat <- utils::getParseData(parse(text=string))
    g <- parser2graph(dat[!(dat$text %in% ignore), ])
    leaves <- V(g)[!degree(g, mode='out')]                             # tree leaves
    preds <- sapply(leaves, neighbors, g=g, mode="in")                 # their predecessors
    vertex_attr(g, 'label', preds) <- vertex_attr(g, 'label', leaves)  # bump labels up a level
    g <- g - leaves                                                    # remove the leaves
    gaps <- V(g)[!nchar(vertex_attr(g, 'label'))]                      # gaps where ()/{} were
    nebs <- c(sapply(gaps, neighbors, graph=g, mode='all'))            # neighbors of gaps
    g <- add_edges(g, nebs, color='blue')                              # edges around the gaps
    g <- g - V(g)[!nchar(vertex_attr(g, 'label'))]                     # remove leaves/gaps
    plot(g, layout=layout.reingold.tilford, ...)
    title(string, cex.main=2.5)
}

An example, slightly more nested expression. The animation shows how original tree is collapsed.

## Example string
library(igraph)
string <- "(x*x)+(x*x+x)*x"

parseTree(string,  # plus some graphing stuff
          vertex.color="#FCFDBFFF", vertex.frame.color=NA,
          vertex.label.font=2, vertex.label.cex=2.5,
          vertex.label.color="darkred", vertex.size=25,
          asp=.7, edge.width=3, margin=-.05)

